<?php

namespace Drupal\system_monitor_examples\Commands;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\system_monitor\Logger\SystemMonitorLoggerInterface;
use Drupal\system_monitor\SystemMonitorLogLevel;
use Drupal\system_monitor\SystemMonitorTaskManager;
use Drupal\system_monitor\SystemMonitorUtility;
use Drush\Commands\DrushCommands;

/**
 * System monitor drush commands.
 */
class SystemMonitorExamplesCommands extends DrushCommands {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The system monitor logger interface.
   *
   * @var \Drupal\system_monitor\Logger\SystemMonitorLoggerInterface
   */
  protected $systemMonitorLogger;

  /**
   * The system monitor task manager.
   *
   * @var \Drupal\system_monitor\SystemMonitorTaskManager
   */
  protected $systemMonitorManager;

  /**
   * The system monitor utility.
   *
   * @var \Drupal\system_monitor\SystemMonitorUtility
   */
  protected $systemMonitorUtility;

  /**
   * SystemMonitorCommands constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The connection to the database.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\system_monitor\Logger\SystemMonitorLoggerInterface $system_monitor_logger
   *   System monitor task manager.
   * @param \Drupal\system_monitor\SystemMonitorTaskManager $system_monitor_manager
   *   System monitor task manager.
   * @param \Drupal\system_monitor\SystemMonitorUtility $system_monitor_utility
   *   System monitor utility.
   */
  public function __construct(Connection $database, EntityTypeManagerInterface $entity_type_manager, SystemMonitorLoggerInterface $system_monitor_logger, SystemMonitorTaskManager $system_monitor_manager, SystemMonitorUtility $system_monitor_utility) {
    parent::__construct();
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->systemMonitorLogger = $system_monitor_logger;
    $this->systemMonitorManager = $system_monitor_manager;
    $this->systemMonitorUtility = $system_monitor_utility;
  }

  /**
   * Generate sample log data.
   *
   * @command sme:gl
   * @aliases sme-gen-logs
   */
  public function generateLogs() {
    // Clear out the current system monitor database logs.
    $this->database->truncate('system_monitor_log')->execute();

    $active_monitor_ids = $this->systemMonitorUtility->getActiveMonitorIds();
    $active_monitors = $this->entityTypeManager->getStorage('system_monitor')->loadMultiple($active_monitor_ids);
    $log_levels = $this->systemMonitorLogger->getRfcToSystemMonitorMap();

    foreach ($active_monitors as $active_monitor) {
      $tasks = $this->systemMonitorManager->getAvailableTasksByMonitor($active_monitor, [], TRUE);
      $first_task = TRUE;
      foreach ($tasks as $task => $task_label) {
        $num_log = 120;
        $logs = [];
        for ($days_ago = $num_log; $days_ago >= 0; $days_ago--) {
          $timestamp = strtotime("-{$days_ago} days") + rand(1, 10);
          // Always make the last entry for the first task be "unhealthy".
          if ($days_ago === 0 && $first_task) {
            $status = 1;
            $first_task = FALSE;
          }
          else {
            $status = (is_int($days_ago / 3) || is_int($days_ago / 4)) ? 6 : 1 + 3 * rand(0, 2);
          }
          $logs[$timestamp] = [
            'status' => $status,
            'values' => [
              'monitor_task_name' => $task,
              'monitor_type' => $active_monitor->id(),
              'description' => 'Description of the event that occured for the ' . $task_label . ' task.',
              'timestamp' => $timestamp,
              'first_report' => 0,
            ],
          ];
        }
        ksort($logs);
        $ok_status = SystemMonitorLogLevel::STATUS_OK;
        $previous_status = $ok_status;
        foreach ($logs as $log) {
          $log_status = $log_levels[$log['status']];
          if ($log_status > $ok_status && $previous_status == $ok_status) {
            $log['values']['first_report'] = 1;
          }
          $previous_status = $log_status;
          $this->systemMonitorLogger->log($log['status'], 'Test Message', $log['values']);
        }
      }
    }
    Cache::invalidateTags(['system_monitor', 'sm_status_page']);
  }

}
